--[[

PremadeRater 1.05

Rates Battleground instances upon entry so you can easily tell if you're
fighting a premade team, and should take the AB proc instead.

©2009, Sydnius. All Rights Reserved.

]]

local PREMADERATERVERSION="1.05"
local eventFrame=CreateFrame("Frame","myPremadeRaterFrame")
local timer=CreateFrame("Frame","myPremadeRaterTimer")
local displayFrame=CreateFrame("Frame","myPremadeRaterDisplay",UIParent)
local dropDownFrame=CreateFrame("Frame","PremadeRaterDropDown",displayFrame,"UIDropDownMenuTemplate")
local hordeIcon=displayFrame:CreateTexture("hordeIcon","OVERLAY")
local hordeDisplayNum=displayFrame:CreateFontString(nil,"OVERLAY")
local hordeDisplayRealm=displayFrame:CreateFontString(nil,"OVERLAY")
local allianceIcon=displayFrame:CreateTexture("allianceIcon","OVERLAY")
local allianceDisplayNum=displayFrame:CreateFontString(nil,"OVERLAY")
local allianceDisplayRealm=displayFrame:CreateFontString(nil,"OVERLAY")
local counter
local counting
local myRealm=GetRealmName()
local myFaction
local myGetTime=GetTime
local inbg=false
local worldStateFrameWasUp=false
PremadeRater={}
SLASH_PREMADERATER1="/premaderater"


function PremadeRater:Initialize()
	SydUtils:EnableLogging(self,'PremadeRater '..PREMADERATERVERSION..': ',1,0,0)
	
	eventFrame:SetScript("OnEvent",PremadeRater.OnEvent)
	eventFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	eventFrame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE")
	eventFrame:RegisterEvent("VARIABLES_LOADED")
end

function PremadeRater.Load()
	PremadeRater:Log('Loaded.')

	SlashCmdList["PREMADERATER"]=PremadeRater.SlashCmdHandler
	
	timer:Hide()
	timer:SetScript("OnUpdate",PremadeRater.Timer)
	
	displayFrame:SetHeight(40)
	displayFrame:SetWidth(160)
	displayFrame:SetBackdrop({bgFile=[[Interface\Tooltips\UI-Tooltip-Background]],
				edgeFile=[[Interface\Tooltips\\UI-Tooltip-Border]],
				tile=true,tileSize=16,edgeSize=16,
				insets={left=5,right=5,top=5,bottom=5} })
	displayFrame:SetBackdropColor(1,1,1,0)
	displayFrame:SetBackdropBorderColor(1,1,1,0)
	displayFrame:SetFrameStrata("MEDIUM")
	displayFrame:SetFrameLevel(1)
	displayFrame:SetClampedToScreen(true)
	displayFrame:ClearAllPoints()

	if not premadeRaterVars then
		--first time run, setup defaults
		PremadeRater:Log('Initializing Saved Variables')
		premadeRaterVars={}
		premadeRaterVars["version"]=PREMADERATERVERSION
		premadeRaterVars["movable"]=true
		displayFrame:SetMovable(true)
		displayFrame:SetPoint("CENTER",UIParent,"CENTER",0,0)
		premadeRaterVars["displayFramePosX"],premadeRaterVars["displayFramePosY"]=displayFrame:GetLeft(),displayFrame:GetBottom()
	else
		PremadeRater:Log('Saved Variables Loaded')
		displayFrame:SetMovable(premadeRaterVars["movable"])
		displayFrame:SetPoint("BOTTOMLEFT",UIParent,premadeRaterVars["displayFramePosX"],premadeRaterVars["displayFramePosY"])
	end

	allianceIcon:SetTexture[[Interface\WorldStateFrame\AllianceIcon.blp]]
	hordeIcon:SetTexture[[Interface\WorldStateFrame\HordeIcon.blp]]
	allianceIcon:SetHeight(16)
	allianceIcon:SetWidth(16)
	allianceIcon:SetPoint("CENTER",displayFrame,"CENTER",-50,8)
	hordeIcon:SetHeight(16)
	hordeIcon:SetWidth(16)
	hordeIcon:SetPoint("CENTER",displayFrame,"CENTER",-50,-8)
	hordeDisplayNum:SetPoint("TOPRIGHT",displayFrame,"CENTER",-58,-3)
	hordeDisplayNum:SetFontObject("GameFontNormalSmall")
	hordeDisplayNum:SetText("-")
	hordeDisplayRealm:SetPoint("TOPLEFT",displayFrame,"CENTER",-40,-3)
	hordeDisplayRealm:SetFontObject("GameFontNormalSmall")
	hordeDisplayRealm:SetText("-")
	allianceDisplayNum:SetPoint("BOTTOMRIGHT",displayFrame,"CENTER",-58,3)
	allianceDisplayNum:SetFontObject("GameFontNormalSmall")
	allianceDisplayNum:SetText("-")
	allianceDisplayRealm:SetPoint("BOTTOMLEFT",displayFrame,"CENTER",-40,3)
	allianceDisplayRealm:SetFontObject("GameFontNormalSmall")
	allianceDisplayRealm:SetText("-")

	displayFrame:EnableMouse()
	displayFrame:SetScript("OnMouseDown",PremadeRater.handleMouseDown)
	displayFrame:SetScript("OnMouseUp",PremadeRater.handleMouseUp)
	UIDropDownMenu_Initialize(dropDownFrame,PremadeRater.handleDropDown)
	

	myFaction=UnitFactionGroup("player")
	if myFaction=="Horde" then
		myFaction=0
	else
		myFaction=1
	end

	--in case we crash & relog IN a bg, simulate a zone change event when we load
	PremadeRater.handleZoneChange()
end

function PremadeRater.SlashCmdHandler(msg)
	PremadeRater:Log(msg)
	if msg=="reset" then
		displayFrame:ClearAllPoints()
		displayFrame:SetPoint("CENTER",UIParent,"CENTER",0,0)
		premadeRaterVars["movable"]=true
		displayFrame:SetMovable(true)
		premadeRaterVars["displayFramePosX"],premadeRaterVars["displayFramePosY"]=displayFrame:GetLeft(),displayFrame:GetBottom()
	elseif msg=="test" then
		PremadeRater.frameUpdate(15,"Dark Iron",14,13,"Lightninghoof",3)
	else
		PremadeRater:Msg('Premaderater commands:\
reset: Reset display position')
	end
end

function PremadeRater.OnEvent(this,event,...)
	if(event=="ZONE_CHANGED_NEW_AREA") then
		PremadeRater.handleZoneChange()
	elseif(event=="UPDATE_BATTLEFIELD_SCORE") then
		if counting then PremadeRater.updateStatus() end
	elseif event=="VARIABLES_LOADED" then
		PremadeRater.Load()
	else
		PremadeRater:Log('UNKNOWN EVENT fired')
	end
end

function PremadeRater.Timer(self,elapsed)
	if not counting then
		timer:Hide()
		return
	end

	counter=counter+elapsed

	if counter>5 then
		if not (WorldStateScoreFrame:IsShown()) then
			if worldStateFrameWasUp then
				SetBattlefieldScoreFaction(-1)
				worldStateFrameWasUp=false
				counter=0
				return
			end
			RequestBattlefieldScoreData()
			counter=0
		else
			worldStateFrameWasUp=true
			counter=0
		end
	end
end

function PremadeRater.handleZoneChange()
	if select(2,IsInInstance())=="pvp" then
		inbg=true
		counting=true
		counter=0
		timer:Show()
		displayFrame:Show()
	else
		if inbg then
			displayFrame:Hide()
			hordeDisplayNum:SetText("-")
			hordeDisplayRealm:SetText("-")
			allianceDisplayNum:SetText("-")
			allianceDisplayRealm:SetText("-")
		end
		inbg=false
		timer:Hide()
		counting=false
		displayFrame:Hide()
	end
end
		

function PremadeRater.updateStatus()

	--the worldstatescoreframe tabs will mess up the received data if clicked
	if WorldStateScoreFrame:IsShown() then
		worldStateFrameWasUp=true
		counter=0
	end

	if worldStateFrameWasUp then return end

	local time=myGetTime()
	
	local horde=0
	local alliance=0
	local realmsAlliance={}
	local realmsHorde={}
	local i,v,players,name,faction,class,pos,realm
	
	players=GetNumBattlefieldScores()

	if players==0 then return end
	
	for i=1,players do
		name,_,_,_,_,faction,_,_,class=GetBattlefieldScore(i)
		if not name then break end
		pos=string.find(name,'-')
		if pos~=nil then
			realm=string.sub(name,pos+1,-1)
		else
			realm=myRealm
		end
		
		if faction==1 then
			alliance=alliance+1
			if not realmsAlliance[realm] then realmsAlliance[realm]=1 else realmsAlliance[realm]=realmsAlliance[realm]+1 end
		else
			horde=horde+1
			if not realmsHorde[realm] then realmsHorde[realm]=1 else realmsHorde[realm]=realmsHorde[realm]+1 end
		end
	end

	local maxNumAlliance=0
	local maxRealmAlliance=''
	for v in pairs(realmsAlliance) do
		if realmsAlliance[v]>maxNumAlliance then
			maxRealmAlliance=v
			maxNumAlliance=realmsAlliance[v]
		end
	end

	local maxNumHorde=0
	local maxRealmHorde=''
	for v in pairs(realmsHorde) do
		if realmsHorde[v]>maxNumHorde then
			maxRealmHorde=v
			maxNumHorde=realmsHorde[v]
		end
	end
	
	PremadeRater.frameUpdate(horde,maxRealmHorde,maxNumHorde,alliance,maxRealmAlliance,maxNumAlliance)
end


function PremadeRater.handleMouseDown(this,event,arg1,arg2,...)
	if event=='LeftButton' then
		if premadeRaterVars["movable"] then
			displayFrame:StartMoving()
		end
	end

	if event=='RightButton' then
		ToggleDropDownMenu(1,nil,dropDownFrame,"cursor",0,0)
	end
		
end

function PremadeRater.handleMouseUp(...)
	if premadeRaterVars["movable"] then
		displayFrame:StopMovingOrSizing()
		premadeRaterVars["displayFramePosX"],premadeRaterVars["displayFramePosY"]=displayFrame:GetLeft(),displayFrame:GetBottom()
	end
	
end	

function PremadeRater.handleDropDown(level)
	local info={}
	info.text='PremadeRater '..PREMADERATERVERSION
	info.isTitle=1
	
	UIDropDownMenu_AddButton(info)

	info={}
	if premadeRaterVars["movable"] then
		info.text='Lock'
	else
		info.text='Unlock'
	end

	info.func=PremadeRater.handleDropDownSelection
	UIDropDownMenu_AddButton(info)
end

function PremadeRater.handleDropDownSelection()
	if premadeRaterVars["movable"] then
		premadeRaterVars["movable"]=false
		displayFrame:SetMovable(false)
	else
		premadeRaterVars["movable"]=true
		displayFrame:SetMovable(true)
	end
end

function PremadeRater.frameUpdate(horde,maxRealmHorde,maxHorde,alliance,maxRealmAlliance,maxAlliance)
	if horde~=0 then
		hordeDisplayNum:SetText(horde)
		hordeDisplayRealm:SetText((string.format("%2d",(maxHorde/horde)*100)).."% "..maxRealmHorde)
	else
		hordeDisplayNum:SetText("-")
		hordeDisplayRealm:SetText("-")
	end
	if alliance~=0 then
		allianceDisplayNum:SetText(alliance)
		allianceDisplayRealm:SetText((string.format("%2d",(maxAlliance/alliance)*100)).."% "..maxRealmAlliance)
	else
		allianceDisplayNum:SetText("-")
		allianceDisplayRealm:SetText("-")
	end
end


PremadeRater:Initialize()